@@ -1,6 +1,6 @@
 package Net::SFTP::Foreign::Helpers;
 
-our $VERSION = '1.68_02';
+our $VERSION = '1.68_04';
 
 use strict;
 use warnings;
@@ -26,6 +26,7 @@ our @EXPORT_OK = qw( _is_lnk
                      _is_reg
                      _do_nothing
 		     _glob_to_regex
+                     _file_part
                      _tcroak );
 
 our $debug;
@@ -277,7 +278,7 @@ sub _gen_converter {
             return sub {
                 my $before = length $_[0];
                 $conversion->($_[0]);
-                length $_[0] - $before;
+                length($_[0]) - $before;
             }
         }
         else {
@@ -299,5 +300,11 @@ sub _is_lnk { (0120000 & shift) == 0120000 }
 sub _is_dir { (0040000 & shift) == 0040000 }
 sub _is_reg { (0100000 & shift) == 0100000 }
 
+sub _file_part {
+    my $path = shift;
+    $path =~ m{([^/]*)$} or croak "unable to get file part from path '$path'";
+    $1;
+}
+
 1;
 
